<?php
/**
 *  Invoice4U Wrapper class
 *  Written by Shoshan & Asaf
 *                  2013
 */
include_once(dirname(__FILE__) . '/../db.php');
include_once(dirname(__FILE__) . '/Invoice4U/Invoice4ULoginService.php');
include_once(dirname(__FILE__) . '/Invoice4U/Invoice4UCustomerService.php');
include_once(dirname(__FILE__) . '/Invoice4U/Invoice4UDocumentService.php');
include_once('InvoiceTemplate.php');

/**
 * Represents the languages supported
 * if you want to add another language you also need to change database enum!
 */
abstract class Invoice4ULanguage
{
    const Hebrew = 1;
    const English = 2;
}

/**
 * Represents the document types supported
 */
abstract class Invoice4UDocumentType
{
    const Invoice = 1;
    const Receipt = 2;
    const InvoiceReceipt = 3;
    const InvoiceCredit = 4;
// and here are some unused types:
//    const ProformaInvoice = 5;
//    const InvoiceOrder = 6;
//    const InvoiceQuote = 7;
//    const InvoiceShip = 8;
//    const Deposits = 9;
}

/**
 * Represents the payment types supported
 */
abstract class Invoice4UPaymentType
{
    const CreditCard = 1;
    const Check = 2;
    const MoneyTransfer = 3;
    const Cash = 4;
    const Credit = 5;
}

/**
 * Author: Shoshan & Asaf
 * Date: 31/12/12
 * Time: 16:18
 */
class Invoice4U
{
    const I4U_DOWNLOAD_PREFIX = "http://private.invoice4u.co.il/Views/PDF.aspx?docid=";
    /**
     * @var Invoice4ULoginService
     */
    protected $loginService;
    /**
     * @var Invoice4UCustomerService
     */
    protected $customerService;
    /**
     * @var Invoice4UDocumentService
     */
    protected $documentService;

    /**
     * @var string
     */
    protected $token;

    /**
     * Creates a new instance of the Invoice4U Wrapper class
     * @param string $username
     * @param string $password
     * @throws Exception
     */
    function __construct($username = 'user', $password = 'pass')
    {
        $soapclient_options = array(
            'cache_wsdl' => WSDL_CACHE_DISK,
            'keep_alive'=>true,
        );
        $this->loginService = new Invoice4ULoginService($soapclient_options);
        $ret = $this->loginService->VerifyLogin(new Invoice4UVerifyLogin($username, $password, false));
        $this->token = $ret->VerifyLoginResult;
        if ($this->token === null) {
            throw new Exception("invalid login details.");
        }
        $this->customerService = new Invoice4UCustomerService($soapclient_options);
        $this->documentService = new Invoice4UDocumentService($soapclient_options);
    }
 
	/**
     * @param $Name - The name of the user to be created
     * @param $City - The city of the user to be created
     * @param $Address - The street address of the user to be created
     * @param $Zip - The zip code of the user to be created
     * @param $Cell - The cellular phone number of the user to be created
     * @param $Email - The eMail of the user to be created
     * @param $Phone - The landline phone number of the user to be created
     * @param $Fax - The fax number of the user to be created
     * @return Invoice4UCustomer - The customer object created
     */
    function CreateNewCustomer($Name, $City = null, $Address = null, $Zip = null ,$OrgId = null ,$Cell = null, $Email = null, $Phone = null, $Fax = null)
    {
        $extNumber = $this->GetNextCustomerExtNumber();

        $id = $this->GetNextCustomerId();
        $uid = $this->GetNextCustomerUniqueId();
        $custInfo = new Invoice4UCustomer(true, $Address, $Cell ? $Cell : " ", $City ? $City : " ", $Email ? $Email: " ", /*$extNumber*/null, $Fax ? $Fax : " ", 0,
            $id, $Name, /*1417*/ intval($OrgId), 0, $Phone ? $Phone :" ", false, 0, null, $uid, $Zip ? $Zip : " ");
        $CreateResponse = $this->customerService->Create(new Invoice4UCreate($custInfo, $this->token));
        if (!$CreateResponse->CreateResult || ! $CreateResponse->CreateResult->ExtNumber)
        {
            var_dump($CreateResponse->CreateResult);
            Throw new Exception("Customer creation failed");
        }
        return $CreateResponse->CreateResult;
    }
   
    /**
     * @param InvoicesCurrency $Currency - The currency to be used
     * @param float $Total - The total amount
     * @param bool $AddTax - Whether or not to add tax to the total sent
     * @param float $TaxPercentage - The tax percentage to use
     * @param int $CustomerExtNumber - The customer extension number
     * @param string $Subject - The subject/title of the document
     * @param int $Language - The language of the document
     * @param dateTime $DueDate - The date in which the payment is due
     * @param Invoice4UDocumentType $DocumentType - The type of the document
     * @param string $PDFPath - the relative path to the location on the server
     *                          where you want the documents to be saved(as PDF files)
     * @param float $ConversionRate - The conversion rate to use when converting USD to ILS
     * todo: asaf, $ConversionRate is not really in use, can we remove it?
     * @return string The document unique ID
     */
    function CreateDocument($Currency, $Total, $AddTax, $TaxPercentage , $CustomerExtNumber, $Subject, $Language, $DueDate, $DocumentType, $PDFPath, $ConversionRate = null, $invoice_id = null, $PaidDate = null, $PaymentType = null)
    {
        $cust = $this->GetCustomerByExtNumber($CustomerExtNumber);
        $docItem = new Invoice4UDocumentItem();
        $docItem->Name = $Subject;
        $docItem->Price = $Total;
        $docItem->Quantity = 1;
        $docItems = array();
        $docItems[0] = $docItem;
        $doc = new Invoice4UDocument();
        $doc->Language = $Language;
        $doc->Subject = $Subject;
        $doc->Currency = $Currency;
        $doc->DocumentType = $DocumentType;
        $doc->Total = $Total;
        $doc->ClientID = $cust->ID;
        if ($DocumentType == Invoice4UDocumentType::Invoice)
        {
            $doc->Items = $docItems;
            $doc->PaymentDueDate = $DueDate;
            $doc->TaxIncluded = !$AddTax;
            $doc->TaxPercentage = $TaxPercentage;
            $doc->ConversionRate = $ConversionRate;
        }
        else
        {
            $ref_invoice = self::GetDocumentByDocumentUniqueID($invoice_id);
            $ref_invoice->ReceiptAmount = $ref_invoice->Total;
            $ref_invoice->Paid = $ref_invoice->Total;
            $doc->ReceiptAmount = null;
            $doc->CloseReceipt = true;
            $doc->Total =  null;
            $doc->Invoices = array ($ref_invoice);
            $doc->DocumentReffType = Invoice4UDocumentType::Invoice;
            $doc->Payments = array(new Invoice4UPayment(null,$ref_invoice->Total,null,null,null,null,$PaidDate,null,$ref_invoice,$invoice_id,null,null,1,null,null,$this->GetPaymentTypeByZefoType($PaymentType)));
        }
        // split into 2 lines in order to support php 5.3
        var_dump($doc);
        $document_result = $this->documentService->CreateDocument(new Invoice4UCreateDocument($doc, $this->token));
        print "------------------------------------------------------------<br>";
        var_dump($document_result->CreateDocumentResult->Errors);
        $docId = $document_result->CreateDocumentResult->UniqueID;
        if (!$docId || $docId === "00000000-0000-0000-0000-000000000000")
            Throw new Exception("Document creation failed");
        $remotePath = self::I4U_DOWNLOAD_PREFIX.$docId;
        $localPath = $PDFPath.$docId.".pdf";
        $this->DownloadFileToDirectory($remotePath, $localPath);
        return $docId;
    }

    /**
     * @param string $UniqueID - The unique id of the document to get
     * @return Invoice4UDocument - The document object found
     */
    function GetDocumentByDocumentUniqueID($UniqueID)
    {
        $Response = $this->documentService->GetDocument(new Invoice4UGetDocument($UniqueID,$this->token));
        return $Response->GetDocumentResult;
    }

    private function GetPaymentTypeByZefoType($PaymentType)
    {
        switch ($PaymentType) {
            case 1:
                return Invoice4UPaymentType::Check;
            case 2:
                return Invoice4UPaymentType::MoneyTransfer;
            case 3:
                return Invoice4UPaymentType::Credit;
            case 4:
                return Invoice4UPaymentType::MoneyTransfer;
        }
    }
}

